﻿function New-GrandstreamConfigFile
{
  <#
  .SYNOPSIS
  Командлет предназначен для создания конфигурационных 
  файлов для телефонов Grandstream GXP1610 и GXP2140 
  с модулями расширения клавиатуры GXP2200-EXT
  .DESCRIPTION
  -
  .PARAMETER MACFile
  Путь к файлу со списком MAC-адресов телефонов
  .PARAMETER SetFile
  Путь к файлу с настройками телефонов
  .PARAMETER ExtFile
  Путь к файлу со списком абонентов для быстрого набора
  .PARAMETER CfgDir
  Путь к папке для сохранения файлов конфигурации
  .PARAMETER Passphrase
  Пароль для шифрования файлов конфигурации
  .INPUTS
  -
  .OUTPUTS
  -    
  .NOTES
  Версия 1.1 
  (c) 2019 Александр Галков, alexander@galkov.pro
  .EXAMPLE
  New-GrandstreamConfigFile -MACFile c:\input\mac.csv -SetFile c:\input\set.csv -ExtFile c:\input\ext.csv -CfgDir c:\output -Passphrase some_pass
  .LINK
  www.galkov.pro/deploy_grandstream_ip_phones
  #>

  [CmdletBinding()]
    
  Param
  (
    [Parameter(Mandatory=$true)][string]$MACFile,
    [Parameter(Mandatory=$true)][string]$SetFile,
    [Parameter(Mandatory=$false)][string]$ExtFile,
    [Parameter(Mandatory=$true)][string]$CfgDir,
    [Parameter(Mandatory=$false)][string]$Passphrase
  )

  $mac_addrs = @()
  $mac_lines = Get-Content $MACFile
  for ($i=1; $i -lt $mac_lines.Length ; $i++)
  {
    $mac_addrs += $mac_lines[$i]
  }

  $set_lines = Get-Content $SetFile

  if ($ExtFile)
  {
    $ext_lines = Get-Content $ExtFile
    for ($i=1; $i -lt $ext_lines.Length ; $i++)
    {
      $line = $ext_lines[$i].Split("{;}")
      $ext = $line[0]
      $key = $line[1]
      $acc = $line[2]
      $des = $line[3]
      $val = $line[4]
      $tag_num = 23000+5*($ext-1)
      $set_lines += ';P' + $tag_num++ + ';' + $key + ';' + 'EXT' + $ext + ' Key Mode'
      $set_lines += ';P' + $tag_num++ + ';' + $acc + ';' + 'EXT' + $ext + ' Account'
      $set_lines += ';P' + $tag_num++ + ';' + $des + ";" + 'EXT' + $ext + ' Description'
      $set_lines += ';P' + $tag_num   + ';' + $val + ';' + 'EXT' + $ext + ' Value'
    }
  }

  foreach ($mac_addr in $mac_addrs)
  {
    $xml_doc = New-Object System.XML.XMLDocument
    $xml_dec = $xml_doc.CreateXmlDeclaration("1.0","UTF-8",$null)
    $xml_doc.AppendChild($xml_dec)
    $root_node = $xml_doc.CreateElement("gs_provision")
    $xml_doc.AppendChild($root_node)
    $root_node.SetAttribute("version","1")
    $mac_node = $xml_doc.CreateElement("mac")
    $root_node.AppendChild($mac_node)
    $mac_node.InnerText = $mac_addr
    $cfg_node = $xml_doc.CreateElement("config")
    $root_node.AppendChild($cfg_node)
    $cfg_node.SetAttribute("version","1")
        
    for ($i=1; $i -lt $set_lines.Length ; $i++)
    {
      $line = $set_lines[$i].Split("{;}")
      $mac = $line[0]
      $par = $line[1]
      $val = $line[2]
      $des = $line[3]
      if (($mac -eq '') -or ($mac -eq $mac_addr))
      {
        $com_node = $xml_doc.CreateComment($des)
        $cfg_node.AppendChild($com_node)
        $set_node = $xml_doc.CreateElement($par)
        $cfg_node.AppendChild($set_node)
        $set_node.InnerText = $val
      }       
    }

    $res_file = $CfgDir + "\cfg" + $mac_addr + ".xml"
    if ($Passphrase)
    {
      $tmp_file = $CfgDir + "\cfg" + $mac_addr + "_tmp.xml"
      $xml_doc.Save($tmp_file)
      & 'openssl' enc -e -aes-256-cbc -in $tmp_file -out $res_file -k $Passphrase
      Remove-Item -Path $tmp_file
    }
    else
    {
      $xml_doc.Save($res_file)
    }
  }
}
